@@ -3,6 +3,7 @@
 import asyncio
 import builtins
 import getpass
+import os
 import signal
 import sys
 import threading
@@ -126,6 +127,11 @@ class IPythonKernel(KernelBase):
             compiler_class=XCachingCompiler,
         )
         self.shell.displayhook.session = self.session
+
+        jupyter_session_name = os.environ.get('JPY_SESSION_NAME')
+        if jupyter_session_name:
+            self.shell.user_ns['__file__'] = jupyter_session_name
+
         self.shell.displayhook.pub_socket = self.iopub_socket
         self.shell.displayhook.topic = self._topic("execute_result")
         self.shell.display_pub.session = self.session
@@ -425,10 +431,7 @@ class IPythonKernel(KernelBase):
         finally:
             self._restore_input()
 
-        if res.error_before_exec is not None:
-            err = res.error_before_exec
-        else:
-            err = res.error_in_exec
+        err = res.error_before_exec if res.error_before_exec is not None else res.error_in_exec
 
         if res.success:
             reply_content["status"] = "ok"
